#ifndef _VIDEO_H
#define _VIDEO_H

#include <highgui.h>

#define DEFAULT_VIDEO_FILE "/mnt/backup/cam/red-triangle-up-sequences/traffic-signs-0-xvid-0003-0006.avi"

/* Video stream properties (because cvGetCaptureProperty doesn't work
 * well). */
#define VIDEO_WIDTH 640
#define VIDEO_HEIGHT 480
#define VIDEO_FPS 25

/* Video source type. Each source type has a ID (don't change it).
 * Source type is determined when the programm runs. */
#define VIDEO_FILE_SOURCE 0
#define VIDEO_CAMERA_SOURCE 1

/* Maximum buffer lengths. */
#define FRAMES_MAX 6
#define TIMES_MAX 4 // times buffer for Kalman and fps computing 

/* Saving video and/or images. */
#define SAVEVIDEOS_MAX 2 // SaveVideo structure
#define DISPLAYIMAGES_MAX 2
#define SAVE_IMAGES 1
#define SAVE_VIDEO 2

/* -------------------------------------------------------------------------- */

typedef struct savevideo_t
{
    CvVideoWriter* writer;
    char dir[512]; // directory
    char videoName[512]; // video filename
    char imageName[512]; // image filename
    int imageCounter;
    int request; // != 0 if there is a saving request
    int on; // != 0 if a video is currently written on disk
} SaveVideo;

/* Main video structure. */
typedef struct video_t
{
    CvCapture* capture;
    int numberOfGrabs; // total number of grabs
    int fi; // current frame index = (numberOfGrabs-1) % FRAMES_MAX
    IplImage* frames[FRAMES_MAX]; // grabbed frames
    IplImage* grayscaleFrames[FRAMES_MAX];
    IplImage* display[DISPLAYIMAGES_MAX]; // images for drawing and displaying
    IplImage* mask; // general mask
    
    /* Real times (depend on grabbing speed) */
    int ti; // current time index = (numberOfGrabs-1) % TIMES_MAX
    int64 times[TIMES_MAX]; // times between grabs in ticks
    double realDts[TIMES_MAX]; // times difference between grabs in
                               // seconds
    double fps; // frames per second (real)
    
    /* File times (doesn't depend on grabbing speed, depend on the
     * video file's header.) */
    double fileDts[TIMES_MAX]; // times between grabbed frames in
                               // seconds
    /* VIDEO_FILE_SOURCE or VIDEO_CAMERA_SOURCE. Useful to know which
     * times we have to use (real times or file times) for tracking.
     * dts points to realDts or fileDts according to sourceType. */
    int sourceType;
    double *dts;

    /* Video saving */
    SaveVideo save[SAVEVIDEOS_MAX];
    char inputFileName[512];

    /* Fonts to write on images */
    struct
    {
        CvFont plain;
        CvFont plainBold;
        CvFont plainSmall;
        CvFont plainSmallBold;
    } fonts;
} Video;

/* -------------------------------------------------------------------------- */

/* Initialize, allocate memory and free it. */
int initVideo( Video* video, int argc, char** argv );
int freeVideo( Video* video );
    
/* Return frame from the video file. Each call returns the frame whose
 * number is (last frame number + skipframe). For instance, if
 * skipFrame is 5, it returns frame 0, 5, 10, etc. There are some
 * hacks because cvSetCaptureProperty() doesn't work. If copiedFrame
 * is 0 function works like cvQueryFrame(), else function copies the
 * queried frame into copiedFrame (you must allocate memory before)
 * for subsequent use (usefeful because cvQueryFrame() overwrite old
 * data and return always the same pointer). Don't forget to free the
 * cloned image. */
IplImage* getFrameFromFile( CvCapture* capture, int skipFrame = 1,
                            IplImage* copiedFrame = NULL );

/* Get frame from V4L or FireWire camera. */
IplImage* getFrameFromCam( CvCapture* capture, IplImage* copiedFrame = NULL );

/* Grab a new frame and update data in video structure. skipFrame is
 * the same as in getFrameFromFile(). */
IplImage* videoGrab( Video* video, int skipFrame = 1 );

/* Save frame in a MPEG sequence named inputvideoname-xx/video.mpg
 * where xx is a number between 00 and 99 depending on directories
 * that already exist. Saving is effective only if
 * video->save[i].request != 0. Images inputvideoname-xx/yyyyyy.png
 * can also be saved. saveFlags can be SAVE_IMAGES and/or
 * SAVE_VIDEO. */
void saveVideo( Video* video, SaveVideo* save, IplImage* frame,
                int saveFlags = 0 );
    
#endif
